Imports System.Xml

Public Class BlocksData
    Implements Collections.IEnumerable

    Public Class BlockEnumerator
        Implements Collections.IEnumerator

        Private _MyEnum As IEnumerator

        Public Sub New(ByVal Data As XmlNodeList)
            _MyEnum = Data.GetEnumerator()
        End Sub

        Public ReadOnly Property Current() As Object Implements Collections.IEnumerator.Current
            Get
                Return DirectCast(_MyEnum.Current, XmlElement).Attributes("name").InnerText
            End Get
        End Property

        Public Function MoveNext() As Boolean Implements Collections.IEnumerator.MoveNext
            If _MyEnum.MoveNext Then
                Return True
            Else
                _MyEnum.Reset()
                Return False
            End If
        End Function

        Public Sub Reset() Implements Collections.IEnumerator.Reset
            _MyEnum.Reset()
        End Sub

    End Class

#Region " Private Members "

    Private _Doc As New XmlDocument
    Private _MyEnum As BlockEnumerator
    Private _Init As Boolean = False
    Private Shared _Instance As New BlocksData

    Private Sub New()
    End Sub

#End Region

#Region " Simpleton Interface Exposer "

    Shared ReadOnly Property I() As BlocksData
        Get
            Return _Instance
        End Get
    End Property

#End Region

#Region " Public Members "

    Public Sub Load(ByVal ResName As String)
        Dim Stream As IO.Stream = _
            Reflection.[Assembly].GetExecutingAssembly().GetManifestResourceStream( _
                Me.[GetType], ResName _
            )
        _Doc.Load(Stream)
        Stream.Close()
        _MyEnum = New BlockEnumerator(_Doc.Item("blocks").ChildNodes)
        _Init = True
    End Sub

    Public ReadOnly Property Init() As Boolean
        Get
            Return _Init
        End Get
    End Property

    Public Function FirstChar(ByVal Block As String) As Integer
        For Each BlockNode As XmlElement In _Doc.Item("blocks").ChildNodes
            If BlockNode.Attributes("name").Value.ToLower() = Block.ToLower() Then
                Return _
                    Integer.Parse( _
                        BlockNode.Item("begin").InnerText, _
                        Globalization.NumberStyles.HexNumber _
                    )
            End If
        Next
    End Function

    Public Function LastChar(ByVal Block As String) As Integer
        For Each BlockNode As XmlElement In _Doc.Item("blocks").ChildNodes
            If BlockNode.Attributes("name").Value.ToLower() = Block.ToLower() Then
                Return _
                    Integer.Parse( _
                        BlockNode.Item("end").InnerText, _
                        Globalization.NumberStyles.HexNumber _
                    )
            End If
        Next
    End Function

    Public Function FindBlock(ByVal Character As Char) As String
        Dim CharVal As Integer = AscW(Character)

        For Each BlockNode As XmlElement In _Doc.Item("blocks").ChildNodes
            If _
                Integer.Parse( _
                    BlockNode.Item("begin").InnerText, _
                    Globalization.NumberStyles.HexNumber _
                ) <= CharVal AndAlso _
                Integer.Parse( _
                    BlockNode.Item("end").InnerText, _
                    Globalization.NumberStyles.HexNumber _
                ) >= CharVal _
            Then
                Return BlockNode.Attributes("name").Value
            End If
        Next
    End Function

    Public Function GetEnumerator() As Collections.IEnumerator Implements Collections.IEnumerable.GetEnumerator
        Return _MyEnum
    End Function

#End Region

End Class
