Imports System.Windows.Forms
Imports System.Drawing

Public Class CharChart : Inherits UserControl

#Region " Private Members "

    Private _FirstChar As Integer = &H20
    Private _SelChar As Integer = &H20
    Private _Brush As Brush
    Private _GridColor As Color = Color.Black
    Private _GridPen As Pen = Pens.Black
    Private _GridHighlightColor As Color = Color.Black
    Private _GridHighlightPen As Pen = Pens.Black
    Private _HighlightBackColor As Color = Color.White
    Private _HighlightBackBrush As Brush = Brushes.White
    Private _Columns As Integer = 10
    Private _FontSize As Single
    Private _Grid As Rectangle

#End Region

#Region " Public Events "

    Public Event SelChanged(ByVal NewChar As Char)
    Public Event CharInvoke(ByVal Value As Char)

#End Region

#Region " Public Properties "

    Public Property SelChar() As Char
        Get
            Return ChrW(_SelChar)
        End Get
        Set(ByVal Value As Char)
            _SelChar = Ascw(Value)
            _FirstChar = _SelChar
            VScrollB.Value = _SelChar

            Me.Refresh()
            RaiseEvent SelChanged(Value)
        End Set
    End Property

    Public Overrides Property ForeColor() As Color
        Get
            Return MyBase.ForeColor
        End Get
        Set(ByVal Value As Color)
            MyBase.ForeColor = Value
            _Brush = New SolidBrush(Value)
            Me.Refresh()
        End Set
    End Property

    Public Property GridColor() As Color
        Get
            Return _GridColor
        End Get
        Set(ByVal Value As Color)
            _GridColor = Value
            _GridPen = New Pen(_GridColor)
            Me.Refresh()
        End Set
    End Property

    Public Property GridHighlightColor() As Color
        Get
            Return _GridHighlightColor
        End Get
        Set(ByVal Value As Color)
            _GridHighlightColor = Value
            _GridHighlightPen = New Pen(_GridHighlightColor)
            Me.Refresh()
        End Set
    End Property

    Public Property HighlightBackColor() As Color
        Get
            Return _HighlightBackColor
        End Get
        Set(ByVal Value As Color)
            _HighlightBackColor = Value
            _HighlightBackBrush = New SolidBrush(_HighlightBackColor)
            Me.Refresh()
        End Set
    End Property

    Public Property Columns() As Integer
        Get
            Return _Columns
        End Get
        Set(ByVal Value As Integer)
            _Columns = Value

            Dim ColWidth As Double = (Me.Width - VScrollB.Width - 1) / _Columns
            Dim G As Graphics = Me.CreateGraphics()

            _Grid = New Rectangle( _
                New Point(0, 0), _
                G.MeasureString(ChrW(&H2588), MyBase.Font).ToSize() _
            )
            G.Dispose()

            _Grid.Height = CInt(Math.Floor(ColWidth * ColWidth / _Grid.Width))
            _Grid.Width = CInt(Math.Floor(ColWidth))

            VScrollB.SmallChange = _Columns
            VScrollB.LargeChange = _Columns * (Me.Height \ _Grid.Height)

            Me.Refresh()
        End Set
    End Property

#End Region

#Region " Code gnr par le Concepteur Windows Form "

    Public Sub New()
        MyBase.New()

        'Cet appel est requis par le Concepteur Windows Form.
        InitializeComponent()

        'Ajoutez une initialisation quelconque aprs l'appel InitializeComponent()
    End Sub

    'La mthode substitue Dispose du UserControl pour nettoyer la liste des composants.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Requis par le Concepteur Windows Form
    Private components As System.ComponentModel.IContainer

    'REMARQUE: la procdure suivante est requise par le Concepteur Windows Form
    'Elle peut tre modifie en utilisant le Concepteur Windows Form.  
    'Ne la modifiez pas en utilisant l'diteur de code.
    Friend WithEvents VScrollB As System.Windows.Forms.VScrollBar
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.VScrollB = New System.Windows.Forms.VScrollBar
        Me.SuspendLayout()
        '
        'VScrollB
        '
        Me.VScrollB.Location = New System.Drawing.Point(155, 20)
        Me.VScrollB.Maximum = 65535
        Me.VScrollB.Minimum = 32
        Me.VScrollB.Name = "VScrollB"
        Me.VScrollB.Size = New System.Drawing.Size(20, 95)
        Me.VScrollB.TabIndex = 1
        Me.VScrollB.Value = 32
        '
        'CharChart
        '
        Me.Controls.Add(Me.VScrollB)
        Me.Name = "CharChart"
        Me.Size = New System.Drawing.Size(190, 145)
        Me.ResumeLayout(False)

    End Sub

#End Region

    Private Sub CharChart_Resize(ByVal Sender As Object, ByVal E As EventArgs) Handles MyBase.Resize
        With VScrollB
            .Location = New Point(Me.Width - .Width, 0)
            .Size = New Size(.Width, Me.Height)
        End With

        ' re-calculate column format '
        Columns = _Columns
    End Sub

    Private Sub CharChart_Paint(ByVal Sender As Object, ByVal E As PaintEventArgs) Handles MyBase.Paint
        With E.Graphics
            Dim CurrChar As Integer = _FirstChar
            Dim FontBrush As New SolidBrush(MyBase.ForeColor)

            Dim SelPos As Rectangle
            Dim SelPoint As Point

            For Y As Integer = 0 To Me.Height \ CInt(_Grid.Height)
                For X As Integer = 0 To _Columns - 1
                    Dim DrawRect As New Rectangle( _
                        X * _Grid.Width, Y * _Grid.Height, _
                        _Grid.Width, _Grid.Height _
                    )
                    Dim CharSize As SizeF = .MeasureString(ChrW(CurrChar), MyBase.Font)
                    Dim XPos As Integer = CInt(DrawRect.X + DrawRect.Width / 2 - CharSize.Width / 2)
                    Dim YPos As Integer = CInt(DrawRect.Y + DrawRect.Height / 2 - CharSize.Height / 2)

                    .DrawRectangle(_GridPen, DrawRect)

                    .DrawString( _
                        ChrW(CurrChar), MyBase.Font, FontBrush, _
                        XPos, YPos _
                    )

                    If CurrChar = _SelChar Then
                        SelPos = DrawRect
                        SelPoint = New Point(XPos, YPos)
                    End If

                    If CurrChar > 65535 Then
                        Exit For
                    End If
                    CurrChar += 1
                Next

                If CurrChar > 65535 Then
                    Exit For
                End If
            Next

            If Not SelPos.IsEmpty Then
                .FillRectangle(_HighlightBackBrush, SelPos)
                .DrawRectangle(_GridHighlightPen, SelPos)
                .DrawString(ChrW(_SelChar), MyBase.Font, FontBrush, SelPoint.X, SelPoint.Y)
            End If

            FontBrush.Dispose()
        End With
    End Sub

    Private Sub VScrollB_Scroll(ByVal Sender As Object, ByVal E As ScrollEventArgs) Handles VScrollB.Scroll
        _FirstChar = VScrollB.Value
        Me.Refresh()
    End Sub

    Private Sub CharChart_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown
        If e.KeyCode = Keys.Right Then
            _SelChar = _SelChar + 1
        End If
    End Sub

    Protected Overrides Sub Finalize()
        MyBase.Finalize()

        ' free unmanaged resources '
        _Brush.Dispose()
        _GridPen.Dispose()
        _GridHighlightPen.Dispose()
        _HighlightBackBrush.Dispose()
    End Sub

    Private Sub CharChart_KeyPress(ByVal Sender As Object, ByVal E As KeyPressEventArgs) Handles MyBase.KeyPress

    End Sub

    Private Sub CharChart_MouseUp(ByVal Sender As Object, ByVal E As MouseEventArgs) Handles MyBase.MouseUp
        Dim XPos As Integer = E.X \ _Grid.Width
        Dim YPos As Integer = E.Y \ _Grid.Height

        If XPos >= 0 AndAlso YPos <= _Columns AndAlso YPos >= 0 Then
            _SelChar = _FirstChar + YPos * _Columns + XPos
            Me.Refresh()
            RaiseEvent SelChanged(ChrW(_SelChar))
        End If
    End Sub

    Private Sub CharChart_DoubleClick(ByVal Sender As Object, ByVal E As EventArgs) Handles MyBase.DoubleClick
        RaiseEvent CharInvoke(ChrW(_SelChar))
    End Sub

End Class
